@@ -1162,6 +1162,10 @@ class AutoscalersRestTransport(AutoscalersTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
