@@ -1523,6 +1523,10 @@ class BackendBucketsRestTransport(BackendBucketsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
