@@ -1984,6 +1984,10 @@ class BackendServicesRestTransport(BackendServicesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
