@@ -1058,6 +1058,10 @@ class ExternalVpnGatewaysRestTransport(ExternalVpnGatewaysTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
