@@ -1348,6 +1348,10 @@ class ForwardingRulesRestTransport(ForwardingRulesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
