@@ -735,6 +735,10 @@ class GlobalAddressesRestTransport(GlobalAddressesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
