@@ -1216,6 +1216,10 @@ class GlobalForwardingRulesRestTransport(GlobalForwardingRulesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
