@@ -1178,6 +1178,10 @@ class HealthChecksRestTransport(HealthChecksTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
