@@ -1494,6 +1494,10 @@ class InstanceGroupsRestTransport(InstanceGroupsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
