@@ -1245,6 +1245,10 @@ class InstanceTemplatesRestTransport(InstanceTemplatesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
