@@ -1049,6 +1049,10 @@ class InterconnectAttachmentsRestTransport(InterconnectAttachmentsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
