@@ -1026,6 +1026,10 @@ class InterconnectsRestTransport(InterconnectsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
