@@ -1237,6 +1237,10 @@ class MachineImagesRestTransport(MachineImagesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
