@@ -1538,6 +1538,10 @@ class NetworkEndpointGroupsRestTransport(NetworkEndpointGroupsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
