@@ -2102,6 +2102,10 @@ class NodeGroupsRestTransport(NodeGroupsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
