@@ -1369,6 +1369,10 @@ class NodeTemplatesRestTransport(NodeTemplatesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
