@@ -1190,6 +1190,10 @@ class PacketMirroringsRestTransport(PacketMirroringsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
