@@ -909,6 +909,10 @@ class PublicAdvertisedPrefixesRestTransport(PublicAdvertisedPrefixesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
