@@ -1047,6 +1047,10 @@ class PublicDelegatedPrefixesRestTransport(PublicDelegatedPrefixesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
