@@ -1052,6 +1052,10 @@ class RegionAutoscalersRestTransport(RegionAutoscalersTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
