@@ -1207,6 +1207,10 @@ class RegionBackendServicesRestTransport(RegionBackendServicesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
