@@ -2006,6 +2006,10 @@ class RegionDisksRestTransport(RegionDisksTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
