@@ -908,6 +908,10 @@ class RegionHealthCheckServicesRestTransport(RegionHealthCheckServicesTransport)
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
