@@ -1062,6 +1062,10 @@ class RegionHealthChecksRestTransport(RegionHealthChecksTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
