@@ -762,6 +762,10 @@ class RegionInstanceGroupsRestTransport(RegionInstanceGroupsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
