@@ -729,6 +729,10 @@ class RegionOperationsRestTransport(RegionOperationsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
