@@ -1196,6 +1196,10 @@ class RegionUrlMapsRestTransport(RegionUrlMapsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
