@@ -1382,6 +1382,10 @@ class ResourcePoliciesRestTransport(ResourcePoliciesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
