@@ -1776,6 +1776,10 @@ class SecurityPoliciesRestTransport(SecurityPoliciesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
