@@ -1549,6 +1549,10 @@ class ServiceAttachmentsRestTransport(ServiceAttachmentsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
