@@ -1365,6 +1365,10 @@ class SnapshotsRestTransport(SnapshotsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
