@@ -1011,6 +1011,10 @@ class SslPoliciesRestTransport(SslPoliciesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
