@@ -1968,6 +1968,10 @@ class SubnetworksRestTransport(SubnetworksTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
