@@ -1195,6 +1195,10 @@ class TargetHttpProxiesRestTransport(TargetHttpProxiesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
