@@ -1682,6 +1682,10 @@ class TargetHttpsProxiesRestTransport(TargetHttpsProxiesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
