@@ -1777,6 +1777,10 @@ class TargetPoolsRestTransport(TargetPoolsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
