@@ -1386,6 +1386,10 @@ class TargetSslProxiesRestTransport(TargetSslProxiesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
