@@ -1068,6 +1068,10 @@ class TargetTcpProxiesRestTransport(TargetTcpProxiesTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
