@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ from typing import Dict, Type
 
 from .base import TargetVpnGatewaysTransport
 from .rest import TargetVpnGatewaysRestTransport
+from .rest import TargetVpnGatewaysRestInterceptor
 
 
 # Compile a registry of transports.
@@ -27,4 +28,5 @@ _transport_registry["rest"] = TargetVpnGatewaysRestTransport
 __all__ = (
     "TargetVpnGatewaysTransport",
     "TargetVpnGatewaysRestTransport",
+    "TargetVpnGatewaysRestInterceptor",
 )
