@@ -1446,6 +1446,10 @@ class UrlMapsRestTransport(UrlMapsTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
