@@ -1295,6 +1295,10 @@ class VpnGatewaysRestTransport(VpnGatewaysTransport):
         # In C++ this would require a dynamic_cast
         return stub  # type: ignore
 
+    @property
+    def kind(self) -> str:
+        return "rest"
+
     def close(self):
         self._session.close()
 
