@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,8 @@ from .compute import (
     AccessConfig,
     AddAccessConfigInstanceRequest,
     AddAssociationFirewallPolicyRequest,
+    AddAssociationNetworkFirewallPolicyRequest,
+    AddAssociationRegionNetworkFirewallPolicyRequest,
     AddHealthCheckTargetPoolRequest,
     AddInstancesInstanceGroupRequest,
     AddInstanceTargetPoolRequest,
@@ -38,6 +40,8 @@ from .compute import (
     AddressesScopedList,
     AddressList,
     AddRuleFirewallPolicyRequest,
+    AddRuleNetworkFirewallPolicyRequest,
+    AddRuleRegionNetworkFirewallPolicyRequest,
     AddRuleSecurityPolicyRequest,
     AddSignedUrlKeyBackendBucketRequest,
     AddSignedUrlKeyBackendServiceRequest,
@@ -56,6 +60,7 @@ from .compute import (
     AggregatedListInstancesRequest,
     AggregatedListInterconnectAttachmentsRequest,
     AggregatedListMachineTypesRequest,
+    AggregatedListNetworkEdgeSecurityServicesRequest,
     AggregatedListNetworkEndpointGroupsRequest,
     AggregatedListNodeGroupsRequest,
     AggregatedListNodeTemplatesRequest,
@@ -66,6 +71,7 @@ from .compute import (
     AggregatedListReservationsRequest,
     AggregatedListResourcePoliciesRequest,
     AggregatedListRoutersRequest,
+    AggregatedListSecurityPoliciesRequest,
     AggregatedListServiceAttachmentsRequest,
     AggregatedListSslCertificatesRequest,
     AggregatedListSubnetworksRequest,
@@ -107,6 +113,7 @@ from .compute import (
     BackendBucket,
     BackendBucketCdnPolicy,
     BackendBucketCdnPolicyBypassCacheOnRequestHeader,
+    BackendBucketCdnPolicyCacheKeyPolicy,
     BackendBucketCdnPolicyNegativeCachingPolicy,
     BackendBucketList,
     BackendService,
@@ -114,13 +121,20 @@ from .compute import (
     BackendServiceCdnPolicy,
     BackendServiceCdnPolicyBypassCacheOnRequestHeader,
     BackendServiceCdnPolicyNegativeCachingPolicy,
+    BackendServiceConnectionTrackingPolicy,
     BackendServiceFailoverPolicy,
     BackendServiceGroupHealth,
     BackendServiceIAP,
     BackendServiceList,
+    BackendServiceLocalityLoadBalancingPolicyConfig,
+    BackendServiceLocalityLoadBalancingPolicyConfigCustomPolicy,
+    BackendServiceLocalityLoadBalancingPolicyConfigPolicy,
     BackendServiceLogConfig,
     BackendServiceReference,
     BackendServicesScopedList,
+    BfdPacket,
+    BfdStatus,
+    BfdStatusPacketCounts,
     Binding,
     BulkInsertInstanceRequest,
     BulkInsertInstanceResource,
@@ -130,6 +144,8 @@ from .compute import (
     CacheKeyPolicy,
     CircuitBreakers,
     CloneRulesFirewallPolicyRequest,
+    CloneRulesNetworkFirewallPolicyRequest,
+    CloneRulesRegionNetworkFirewallPolicyRequest,
     Commitment,
     CommitmentAggregatedList,
     CommitmentList,
@@ -176,7 +192,10 @@ from .compute import (
     DeleteInterconnectAttachmentRequest,
     DeleteInterconnectRequest,
     DeleteLicenseRequest,
+    DeleteMachineImageRequest,
+    DeleteNetworkEdgeSecurityServiceRequest,
     DeleteNetworkEndpointGroupRequest,
+    DeleteNetworkFirewallPolicyRequest,
     DeleteNetworkRequest,
     DeleteNodeGroupRequest,
     DeleteNodesNodeGroupRequest,
@@ -193,9 +212,11 @@ from .compute import (
     DeleteRegionHealthCheckServiceRequest,
     DeleteRegionInstanceGroupManagerRequest,
     DeleteRegionNetworkEndpointGroupRequest,
+    DeleteRegionNetworkFirewallPolicyRequest,
     DeleteRegionNotificationEndpointRequest,
     DeleteRegionOperationRequest,
     DeleteRegionOperationResponse,
+    DeleteRegionSecurityPolicyRequest,
     DeleteRegionSslCertificateRequest,
     DeleteRegionTargetHttpProxyRequest,
     DeleteRegionTargetHttpsProxyRequest,
@@ -272,6 +293,7 @@ from .compute import (
     FirewallPolicyRule,
     FirewallPolicyRuleMatcher,
     FirewallPolicyRuleMatcherLayer4Config,
+    FirewallPolicyRuleSecureTag,
     FixedOrPercent,
     ForwardingRule,
     ForwardingRuleAggregatedList,
@@ -282,6 +304,8 @@ from .compute import (
     GetAcceleratorTypeRequest,
     GetAddressRequest,
     GetAssociationFirewallPolicyRequest,
+    GetAssociationNetworkFirewallPolicyRequest,
+    GetAssociationRegionNetworkFirewallPolicyRequest,
     GetAutoscalerRequest,
     GetBackendBucketRequest,
     GetBackendServiceRequest,
@@ -290,6 +314,7 @@ from .compute import (
     GetDiskTypeRequest,
     GetEffectiveFirewallsInstanceRequest,
     GetEffectiveFirewallsNetworkRequest,
+    GetEffectiveFirewallsRegionNetworkFirewallPolicyRequest,
     GetExternalVpnGatewayRequest,
     GetFirewallPolicyRequest,
     GetFirewallRequest,
@@ -312,9 +337,12 @@ from .compute import (
     GetIamPolicyInstanceRequest,
     GetIamPolicyInstanceTemplateRequest,
     GetIamPolicyLicenseRequest,
+    GetIamPolicyMachineImageRequest,
+    GetIamPolicyNetworkFirewallPolicyRequest,
     GetIamPolicyNodeGroupRequest,
     GetIamPolicyNodeTemplateRequest,
     GetIamPolicyRegionDiskRequest,
+    GetIamPolicyRegionNetworkFirewallPolicyRequest,
     GetIamPolicyReservationRequest,
     GetIamPolicyResourcePolicyRequest,
     GetIamPolicyServiceAttachmentRequest,
@@ -331,9 +359,12 @@ from .compute import (
     GetInterconnectRequest,
     GetLicenseCodeRequest,
     GetLicenseRequest,
+    GetMachineImageRequest,
     GetMachineTypeRequest,
     GetNatMappingInfoRoutersRequest,
+    GetNetworkEdgeSecurityServiceRequest,
     GetNetworkEndpointGroupRequest,
+    GetNetworkFirewallPolicyRequest,
     GetNetworkRequest,
     GetNodeGroupRequest,
     GetNodeTemplateRequest,
@@ -352,9 +383,11 @@ from .compute import (
     GetRegionInstanceGroupManagerRequest,
     GetRegionInstanceGroupRequest,
     GetRegionNetworkEndpointGroupRequest,
+    GetRegionNetworkFirewallPolicyRequest,
     GetRegionNotificationEndpointRequest,
     GetRegionOperationRequest,
     GetRegionRequest,
+    GetRegionSecurityPolicyRequest,
     GetRegionSslCertificateRequest,
     GetRegionTargetHttpProxyRequest,
     GetRegionTargetHttpsProxyRequest,
@@ -365,6 +398,8 @@ from .compute import (
     GetRouterRequest,
     GetRouterStatusRouterRequest,
     GetRuleFirewallPolicyRequest,
+    GetRuleNetworkFirewallPolicyRequest,
+    GetRuleRegionNetworkFirewallPolicyRequest,
     GetRuleSecurityPolicyRequest,
     GetScreenshotInstanceRequest,
     GetSecurityPolicyRequest,
@@ -454,7 +489,10 @@ from .compute import (
     InsertInterconnectAttachmentRequest,
     InsertInterconnectRequest,
     InsertLicenseRequest,
+    InsertMachineImageRequest,
+    InsertNetworkEdgeSecurityServiceRequest,
     InsertNetworkEndpointGroupRequest,
+    InsertNetworkFirewallPolicyRequest,
     InsertNetworkRequest,
     InsertNodeGroupRequest,
     InsertNodeTemplateRequest,
@@ -469,7 +507,9 @@ from .compute import (
     InsertRegionHealthCheckServiceRequest,
     InsertRegionInstanceGroupManagerRequest,
     InsertRegionNetworkEndpointGroupRequest,
+    InsertRegionNetworkFirewallPolicyRequest,
     InsertRegionNotificationEndpointRequest,
+    InsertRegionSecurityPolicyRequest,
     InsertRegionSslCertificateRequest,
     InsertRegionTargetHttpProxyRequest,
     InsertRegionTargetHttpsProxyRequest,
@@ -480,6 +520,7 @@ from .compute import (
     InsertRouterRequest,
     InsertSecurityPolicyRequest,
     InsertServiceAttachmentRequest,
+    InsertSnapshotRequest,
     InsertSslCertificateRequest,
     InsertSslPolicyRequest,
     InsertSubnetworkRequest,
@@ -536,6 +577,7 @@ from .compute import (
     InstanceManagedByIgmErrorInstanceActionDetails,
     InstanceManagedByIgmErrorManagedInstanceError,
     InstanceMoveRequest,
+    InstanceParams,
     InstanceProperties,
     InstanceReference,
     InstancesAddResourcePoliciesRequest,
@@ -613,12 +655,14 @@ from .compute import (
     ListInterconnectLocationsRequest,
     ListInterconnectsRequest,
     ListLicensesRequest,
+    ListMachineImagesRequest,
     ListMachineTypesRequest,
     ListManagedInstancesInstanceGroupManagersRequest,
     ListManagedInstancesRegionInstanceGroupManagersRequest,
     ListNetworkEndpointGroupsRequest,
     ListNetworkEndpointsGlobalNetworkEndpointGroupsRequest,
     ListNetworkEndpointsNetworkEndpointGroupsRequest,
+    ListNetworkFirewallPoliciesRequest,
     ListNetworksRequest,
     ListNodeGroupsRequest,
     ListNodesNodeGroupsRequest,
@@ -642,8 +686,10 @@ from .compute import (
     ListRegionInstanceGroupManagersRequest,
     ListRegionInstanceGroupsRequest,
     ListRegionNetworkEndpointGroupsRequest,
+    ListRegionNetworkFirewallPoliciesRequest,
     ListRegionNotificationEndpointsRequest,
     ListRegionOperationsRequest,
+    ListRegionSecurityPoliciesRequest,
     ListRegionsRequest,
     ListRegionSslCertificatesRequest,
     ListRegionTargetHttpProxiesRequest,
@@ -682,6 +728,8 @@ from .compute import (
     LogConfigCounterOptions,
     LogConfigCounterOptionsCustomField,
     LogConfigDataAccessOptions,
+    MachineImage,
+    MachineImageList,
     MachineType,
     MachineTypeAggregatedList,
     MachineTypeList,
@@ -698,6 +746,9 @@ from .compute import (
     MoveInstanceProjectRequest,
     NamedPort,
     Network,
+    NetworkEdgeSecurityService,
+    NetworkEdgeSecurityServiceAggregatedList,
+    NetworkEdgeSecurityServicesScopedList,
     NetworkEndpoint,
     NetworkEndpointGroup,
     NetworkEndpointGroupAggregatedList,
@@ -714,6 +765,7 @@ from .compute import (
     NetworkInterface,
     NetworkList,
     NetworkPeering,
+    NetworkPerformanceConfig,
     NetworkRoutingConfig,
     NetworksAddPeeringRequest,
     NetworksGetEffectiveFirewallsResponse,
@@ -748,6 +800,7 @@ from .compute import (
     OperationList,
     OperationsScopedList,
     OutlierDetection,
+    PacketIntervals,
     PacketMirroring,
     PacketMirroringAggregatedList,
     PacketMirroringFilter,
@@ -771,6 +824,8 @@ from .compute import (
     PatchInstanceGroupManagerRequest,
     PatchInterconnectAttachmentRequest,
     PatchInterconnectRequest,
+    PatchNetworkEdgeSecurityServiceRequest,
+    PatchNetworkFirewallPolicyRequest,
     PatchNetworkRequest,
     PatchNodeGroupRequest,
     PatchPacketMirroringRequest,
@@ -783,9 +838,14 @@ from .compute import (
     PatchRegionHealthCheckRequest,
     PatchRegionHealthCheckServiceRequest,
     PatchRegionInstanceGroupManagerRequest,
+    PatchRegionNetworkFirewallPolicyRequest,
+    PatchRegionSecurityPolicyRequest,
+    PatchRegionTargetHttpsProxyRequest,
     PatchRegionUrlMapRequest,
     PatchRouterRequest,
     PatchRuleFirewallPolicyRequest,
+    PatchRuleNetworkFirewallPolicyRequest,
+    PatchRuleRegionNetworkFirewallPolicyRequest,
     PatchRuleSecurityPolicyRequest,
     PatchSecurityPolicyRequest,
     PatchServiceAttachmentRequest,
@@ -847,11 +907,15 @@ from .compute import (
     RegionInstanceGroupsListInstancesRequest,
     RegionInstanceGroupsSetNamedPortsRequest,
     RegionList,
+    RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponse,
+    RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponseEffectiveFirewallPolicy,
     RegionSetLabelsRequest,
     RegionSetPolicyRequest,
     RegionTargetHttpsProxiesSetSslCertificatesRequest,
     RegionUrlMapsValidateRequest,
     RemoveAssociationFirewallPolicyRequest,
+    RemoveAssociationNetworkFirewallPolicyRequest,
+    RemoveAssociationRegionNetworkFirewallPolicyRequest,
     RemoveHealthCheckTargetPoolRequest,
     RemoveInstancesInstanceGroupRequest,
     RemoveInstanceTargetPoolRequest,
@@ -860,6 +924,8 @@ from .compute import (
     RemoveResourcePoliciesInstanceRequest,
     RemoveResourcePoliciesRegionDiskRequest,
     RemoveRuleFirewallPolicyRequest,
+    RemoveRuleNetworkFirewallPolicyRequest,
+    RemoveRuleRegionNetworkFirewallPolicyRequest,
     RemoveRuleSecurityPolicyRequest,
     RequestMirrorPolicy,
     Reservation,
@@ -893,6 +959,7 @@ from .compute import (
     ResourcePolicySnapshotSchedulePolicySnapshotProperties,
     ResourcePolicyWeeklyCycle,
     ResourcePolicyWeeklyCycleDayOfWeek,
+    ResumeInstanceRequest,
     Route,
     RouteAsPath,
     RouteList,
@@ -917,22 +984,33 @@ from .compute import (
     RouterStatusNatStatusNatRuleStatus,
     RouterStatusResponse,
     Rule,
+    SavedAttachedDisk,
+    SavedDisk,
     ScalingScheduleStatus,
     Scheduling,
     SchedulingNodeAffinity,
     ScratchDisks,
     Screenshot,
+    SecurityPoliciesAggregatedList,
     SecurityPoliciesListPreconfiguredExpressionSetsResponse,
+    SecurityPoliciesScopedList,
     SecurityPoliciesWafConfig,
     SecurityPolicy,
     SecurityPolicyAdaptiveProtectionConfig,
     SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig,
     SecurityPolicyAdvancedOptionsConfig,
+    SecurityPolicyDdosProtectionConfig,
     SecurityPolicyList,
+    SecurityPolicyRecaptchaOptionsConfig,
     SecurityPolicyReference,
     SecurityPolicyRule,
+    SecurityPolicyRuleHttpHeaderAction,
+    SecurityPolicyRuleHttpHeaderActionHttpHeaderOption,
     SecurityPolicyRuleMatcher,
     SecurityPolicyRuleMatcherConfig,
+    SecurityPolicyRuleRateLimitOptions,
+    SecurityPolicyRuleRateLimitOptionsThreshold,
+    SecurityPolicyRuleRedirectOptions,
     SecuritySettings,
     SendDiagnosticInterruptInstanceRequest,
     SendDiagnosticInterruptInstanceResponse,
@@ -952,15 +1030,20 @@ from .compute import (
     SetDefaultNetworkTierProjectRequest,
     SetDeletionProtectionInstanceRequest,
     SetDiskAutoDeleteInstanceRequest,
+    SetEdgeSecurityPolicyBackendBucketRequest,
+    SetEdgeSecurityPolicyBackendServiceRequest,
     SetIamPolicyDiskRequest,
     SetIamPolicyFirewallPolicyRequest,
     SetIamPolicyImageRequest,
     SetIamPolicyInstanceRequest,
     SetIamPolicyInstanceTemplateRequest,
     SetIamPolicyLicenseRequest,
+    SetIamPolicyMachineImageRequest,
+    SetIamPolicyNetworkFirewallPolicyRequest,
     SetIamPolicyNodeGroupRequest,
     SetIamPolicyNodeTemplateRequest,
     SetIamPolicyRegionDiskRequest,
+    SetIamPolicyRegionNetworkFirewallPolicyRequest,
     SetIamPolicyReservationRequest,
     SetIamPolicyResourcePolicyRequest,
     SetIamPolicyServiceAttachmentRequest,
@@ -1007,6 +1090,8 @@ from .compute import (
     SetUrlMapTargetHttpProxyRequest,
     SetUrlMapTargetHttpsProxyRequest,
     SetUsageExportBucketProjectRequest,
+    ShareSettings,
+    ShareSettingsProjectConfig,
     ShieldedInstanceConfig,
     ShieldedInstanceIdentity,
     ShieldedInstanceIdentityEntry,
@@ -1015,7 +1100,9 @@ from .compute import (
     SimulateMaintenanceEventInstanceRequest,
     Snapshot,
     SnapshotList,
+    SourceDiskEncryptionKey,
     SourceInstanceParams,
+    SourceInstanceProperties,
     SslCertificate,
     SslCertificateAggregatedList,
     SslCertificateList,
@@ -1042,6 +1129,7 @@ from .compute import (
     SubnetworksScopedList,
     SubnetworksSetPrivateIpGoogleAccessRequest,
     Subsetting,
+    SuspendInstanceRequest,
     SwitchToCustomModeNetworkRequest,
     Tags,
     TargetGrpcProxy,
@@ -1093,11 +1181,14 @@ from .compute import (
     TestIamPermissionsInstanceTemplateRequest,
     TestIamPermissionsLicenseCodeRequest,
     TestIamPermissionsLicenseRequest,
+    TestIamPermissionsMachineImageRequest,
     TestIamPermissionsNetworkEndpointGroupRequest,
+    TestIamPermissionsNetworkFirewallPolicyRequest,
     TestIamPermissionsNodeGroupRequest,
     TestIamPermissionsNodeTemplateRequest,
     TestIamPermissionsPacketMirroringRequest,
     TestIamPermissionsRegionDiskRequest,
+    TestIamPermissionsRegionNetworkFirewallPolicyRequest,
     TestIamPermissionsReservationRequest,
     TestIamPermissionsResourcePolicyRequest,
     TestIamPermissionsServiceAttachmentRequest,
@@ -1121,8 +1212,10 @@ from .compute import (
     UpdatePerInstanceConfigsRegionInstanceGroupManagerRequest,
     UpdateRegionAutoscalerRequest,
     UpdateRegionBackendServiceRequest,
+    UpdateRegionCommitmentRequest,
     UpdateRegionHealthCheckRequest,
     UpdateRegionUrlMapRequest,
+    UpdateReservationRequest,
     UpdateRouterRequest,
     UpdateShieldedInstanceConfigInstanceRequest,
     UpdateUrlMapRequest,
@@ -1188,6 +1281,8 @@ __all__ = (
     "AccessConfig",
     "AddAccessConfigInstanceRequest",
     "AddAssociationFirewallPolicyRequest",
+    "AddAssociationNetworkFirewallPolicyRequest",
+    "AddAssociationRegionNetworkFirewallPolicyRequest",
     "AddHealthCheckTargetPoolRequest",
     "AddInstancesInstanceGroupRequest",
     "AddInstanceTargetPoolRequest",
@@ -1201,6 +1296,8 @@ __all__ = (
     "AddressesScopedList",
     "AddressList",
     "AddRuleFirewallPolicyRequest",
+    "AddRuleNetworkFirewallPolicyRequest",
+    "AddRuleRegionNetworkFirewallPolicyRequest",
     "AddRuleSecurityPolicyRequest",
     "AddSignedUrlKeyBackendBucketRequest",
     "AddSignedUrlKeyBackendServiceRequest",
@@ -1219,6 +1316,7 @@ __all__ = (
     "AggregatedListInstancesRequest",
     "AggregatedListInterconnectAttachmentsRequest",
     "AggregatedListMachineTypesRequest",
+    "AggregatedListNetworkEdgeSecurityServicesRequest",
     "AggregatedListNetworkEndpointGroupsRequest",
     "AggregatedListNodeGroupsRequest",
     "AggregatedListNodeTemplatesRequest",
@@ -1229,6 +1327,7 @@ __all__ = (
     "AggregatedListReservationsRequest",
     "AggregatedListResourcePoliciesRequest",
     "AggregatedListRoutersRequest",
+    "AggregatedListSecurityPoliciesRequest",
     "AggregatedListServiceAttachmentsRequest",
     "AggregatedListSslCertificatesRequest",
     "AggregatedListSubnetworksRequest",
@@ -1270,6 +1369,7 @@ __all__ = (
     "BackendBucket",
     "BackendBucketCdnPolicy",
     "BackendBucketCdnPolicyBypassCacheOnRequestHeader",
+    "BackendBucketCdnPolicyCacheKeyPolicy",
     "BackendBucketCdnPolicyNegativeCachingPolicy",
     "BackendBucketList",
     "BackendService",
@@ -1277,13 +1377,20 @@ __all__ = (
     "BackendServiceCdnPolicy",
     "BackendServiceCdnPolicyBypassCacheOnRequestHeader",
     "BackendServiceCdnPolicyNegativeCachingPolicy",
+    "BackendServiceConnectionTrackingPolicy",
     "BackendServiceFailoverPolicy",
     "BackendServiceGroupHealth",
     "BackendServiceIAP",
     "BackendServiceList",
+    "BackendServiceLocalityLoadBalancingPolicyConfig",
+    "BackendServiceLocalityLoadBalancingPolicyConfigCustomPolicy",
+    "BackendServiceLocalityLoadBalancingPolicyConfigPolicy",
     "BackendServiceLogConfig",
     "BackendServiceReference",
     "BackendServicesScopedList",
+    "BfdPacket",
+    "BfdStatus",
+    "BfdStatusPacketCounts",
     "Binding",
     "BulkInsertInstanceRequest",
     "BulkInsertInstanceResource",
@@ -1293,6 +1400,8 @@ __all__ = (
     "CacheKeyPolicy",
     "CircuitBreakers",
     "CloneRulesFirewallPolicyRequest",
+    "CloneRulesNetworkFirewallPolicyRequest",
+    "CloneRulesRegionNetworkFirewallPolicyRequest",
     "Commitment",
     "CommitmentAggregatedList",
     "CommitmentList",
@@ -1339,7 +1448,10 @@ __all__ = (
     "DeleteInterconnectAttachmentRequest",
     "DeleteInterconnectRequest",
     "DeleteLicenseRequest",
+    "DeleteMachineImageRequest",
+    "DeleteNetworkEdgeSecurityServiceRequest",
     "DeleteNetworkEndpointGroupRequest",
+    "DeleteNetworkFirewallPolicyRequest",
     "DeleteNetworkRequest",
     "DeleteNodeGroupRequest",
     "DeleteNodesNodeGroupRequest",
@@ -1356,9 +1468,11 @@ __all__ = (
     "DeleteRegionHealthCheckServiceRequest",
     "DeleteRegionInstanceGroupManagerRequest",
     "DeleteRegionNetworkEndpointGroupRequest",
+    "DeleteRegionNetworkFirewallPolicyRequest",
     "DeleteRegionNotificationEndpointRequest",
     "DeleteRegionOperationRequest",
     "DeleteRegionOperationResponse",
+    "DeleteRegionSecurityPolicyRequest",
     "DeleteRegionSslCertificateRequest",
     "DeleteRegionTargetHttpProxyRequest",
     "DeleteRegionTargetHttpsProxyRequest",
@@ -1435,6 +1549,7 @@ __all__ = (
     "FirewallPolicyRule",
     "FirewallPolicyRuleMatcher",
     "FirewallPolicyRuleMatcherLayer4Config",
+    "FirewallPolicyRuleSecureTag",
     "FixedOrPercent",
     "ForwardingRule",
     "ForwardingRuleAggregatedList",
@@ -1445,6 +1560,8 @@ __all__ = (
     "GetAcceleratorTypeRequest",
     "GetAddressRequest",
     "GetAssociationFirewallPolicyRequest",
+    "GetAssociationNetworkFirewallPolicyRequest",
+    "GetAssociationRegionNetworkFirewallPolicyRequest",
     "GetAutoscalerRequest",
     "GetBackendBucketRequest",
     "GetBackendServiceRequest",
@@ -1453,6 +1570,7 @@ __all__ = (
     "GetDiskTypeRequest",
     "GetEffectiveFirewallsInstanceRequest",
     "GetEffectiveFirewallsNetworkRequest",
+    "GetEffectiveFirewallsRegionNetworkFirewallPolicyRequest",
     "GetExternalVpnGatewayRequest",
     "GetFirewallPolicyRequest",
     "GetFirewallRequest",
@@ -1475,9 +1593,12 @@ __all__ = (
     "GetIamPolicyInstanceRequest",
     "GetIamPolicyInstanceTemplateRequest",
     "GetIamPolicyLicenseRequest",
+    "GetIamPolicyMachineImageRequest",
+    "GetIamPolicyNetworkFirewallPolicyRequest",
     "GetIamPolicyNodeGroupRequest",
     "GetIamPolicyNodeTemplateRequest",
     "GetIamPolicyRegionDiskRequest",
+    "GetIamPolicyRegionNetworkFirewallPolicyRequest",
     "GetIamPolicyReservationRequest",
     "GetIamPolicyResourcePolicyRequest",
     "GetIamPolicyServiceAttachmentRequest",
@@ -1494,9 +1615,12 @@ __all__ = (
     "GetInterconnectRequest",
     "GetLicenseCodeRequest",
     "GetLicenseRequest",
+    "GetMachineImageRequest",
     "GetMachineTypeRequest",
     "GetNatMappingInfoRoutersRequest",
+    "GetNetworkEdgeSecurityServiceRequest",
     "GetNetworkEndpointGroupRequest",
+    "GetNetworkFirewallPolicyRequest",
     "GetNetworkRequest",
     "GetNodeGroupRequest",
     "GetNodeTemplateRequest",
@@ -1515,9 +1639,11 @@ __all__ = (
     "GetRegionInstanceGroupManagerRequest",
     "GetRegionInstanceGroupRequest",
     "GetRegionNetworkEndpointGroupRequest",
+    "GetRegionNetworkFirewallPolicyRequest",
     "GetRegionNotificationEndpointRequest",
     "GetRegionOperationRequest",
     "GetRegionRequest",
+    "GetRegionSecurityPolicyRequest",
     "GetRegionSslCertificateRequest",
     "GetRegionTargetHttpProxyRequest",
     "GetRegionTargetHttpsProxyRequest",
@@ -1528,6 +1654,8 @@ __all__ = (
     "GetRouterRequest",
     "GetRouterStatusRouterRequest",
     "GetRuleFirewallPolicyRequest",
+    "GetRuleNetworkFirewallPolicyRequest",
+    "GetRuleRegionNetworkFirewallPolicyRequest",
     "GetRuleSecurityPolicyRequest",
     "GetScreenshotInstanceRequest",
     "GetSecurityPolicyRequest",
@@ -1617,7 +1745,10 @@ __all__ = (
     "InsertInterconnectAttachmentRequest",
     "InsertInterconnectRequest",
     "InsertLicenseRequest",
+    "InsertMachineImageRequest",
+    "InsertNetworkEdgeSecurityServiceRequest",
     "InsertNetworkEndpointGroupRequest",
+    "InsertNetworkFirewallPolicyRequest",
     "InsertNetworkRequest",
     "InsertNodeGroupRequest",
     "InsertNodeTemplateRequest",
@@ -1632,7 +1763,9 @@ __all__ = (
     "InsertRegionHealthCheckServiceRequest",
     "InsertRegionInstanceGroupManagerRequest",
     "InsertRegionNetworkEndpointGroupRequest",
+    "InsertRegionNetworkFirewallPolicyRequest",
     "InsertRegionNotificationEndpointRequest",
+    "InsertRegionSecurityPolicyRequest",
     "InsertRegionSslCertificateRequest",
     "InsertRegionTargetHttpProxyRequest",
     "InsertRegionTargetHttpsProxyRequest",
@@ -1643,6 +1776,7 @@ __all__ = (
     "InsertRouterRequest",
     "InsertSecurityPolicyRequest",
     "InsertServiceAttachmentRequest",
+    "InsertSnapshotRequest",
     "InsertSslCertificateRequest",
     "InsertSslPolicyRequest",
     "InsertSubnetworkRequest",
@@ -1699,6 +1833,7 @@ __all__ = (
     "InstanceManagedByIgmErrorInstanceActionDetails",
     "InstanceManagedByIgmErrorManagedInstanceError",
     "InstanceMoveRequest",
+    "InstanceParams",
     "InstanceProperties",
     "InstanceReference",
     "InstancesAddResourcePoliciesRequest",
@@ -1776,12 +1911,14 @@ __all__ = (
     "ListInterconnectLocationsRequest",
     "ListInterconnectsRequest",
     "ListLicensesRequest",
+    "ListMachineImagesRequest",
     "ListMachineTypesRequest",
     "ListManagedInstancesInstanceGroupManagersRequest",
     "ListManagedInstancesRegionInstanceGroupManagersRequest",
     "ListNetworkEndpointGroupsRequest",
     "ListNetworkEndpointsGlobalNetworkEndpointGroupsRequest",
     "ListNetworkEndpointsNetworkEndpointGroupsRequest",
+    "ListNetworkFirewallPoliciesRequest",
     "ListNetworksRequest",
     "ListNodeGroupsRequest",
     "ListNodesNodeGroupsRequest",
@@ -1805,8 +1942,10 @@ __all__ = (
     "ListRegionInstanceGroupManagersRequest",
     "ListRegionInstanceGroupsRequest",
     "ListRegionNetworkEndpointGroupsRequest",
+    "ListRegionNetworkFirewallPoliciesRequest",
     "ListRegionNotificationEndpointsRequest",
     "ListRegionOperationsRequest",
+    "ListRegionSecurityPoliciesRequest",
     "ListRegionsRequest",
     "ListRegionSslCertificatesRequest",
     "ListRegionTargetHttpProxiesRequest",
@@ -1845,6 +1984,8 @@ __all__ = (
     "LogConfigCounterOptions",
     "LogConfigCounterOptionsCustomField",
     "LogConfigDataAccessOptions",
+    "MachineImage",
+    "MachineImageList",
     "MachineType",
     "MachineTypeAggregatedList",
     "MachineTypeList",
@@ -1861,6 +2002,9 @@ __all__ = (
     "MoveInstanceProjectRequest",
     "NamedPort",
     "Network",
+    "NetworkEdgeSecurityService",
+    "NetworkEdgeSecurityServiceAggregatedList",
+    "NetworkEdgeSecurityServicesScopedList",
     "NetworkEndpoint",
     "NetworkEndpointGroup",
     "NetworkEndpointGroupAggregatedList",
@@ -1877,6 +2021,7 @@ __all__ = (
     "NetworkInterface",
     "NetworkList",
     "NetworkPeering",
+    "NetworkPerformanceConfig",
     "NetworkRoutingConfig",
     "NetworksAddPeeringRequest",
     "NetworksGetEffectiveFirewallsResponse",
@@ -1911,6 +2056,7 @@ __all__ = (
     "OperationList",
     "OperationsScopedList",
     "OutlierDetection",
+    "PacketIntervals",
     "PacketMirroring",
     "PacketMirroringAggregatedList",
     "PacketMirroringFilter",
@@ -1934,6 +2080,8 @@ __all__ = (
     "PatchInstanceGroupManagerRequest",
     "PatchInterconnectAttachmentRequest",
     "PatchInterconnectRequest",
+    "PatchNetworkEdgeSecurityServiceRequest",
+    "PatchNetworkFirewallPolicyRequest",
     "PatchNetworkRequest",
     "PatchNodeGroupRequest",
     "PatchPacketMirroringRequest",
@@ -1946,9 +2094,14 @@ __all__ = (
     "PatchRegionHealthCheckRequest",
     "PatchRegionHealthCheckServiceRequest",
     "PatchRegionInstanceGroupManagerRequest",
+    "PatchRegionNetworkFirewallPolicyRequest",
+    "PatchRegionSecurityPolicyRequest",
+    "PatchRegionTargetHttpsProxyRequest",
     "PatchRegionUrlMapRequest",
     "PatchRouterRequest",
     "PatchRuleFirewallPolicyRequest",
+    "PatchRuleNetworkFirewallPolicyRequest",
+    "PatchRuleRegionNetworkFirewallPolicyRequest",
     "PatchRuleSecurityPolicyRequest",
     "PatchSecurityPolicyRequest",
     "PatchServiceAttachmentRequest",
@@ -2010,11 +2163,15 @@ __all__ = (
     "RegionInstanceGroupsListInstancesRequest",
     "RegionInstanceGroupsSetNamedPortsRequest",
     "RegionList",
+    "RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponse",
+    "RegionNetworkFirewallPoliciesGetEffectiveFirewallsResponseEffectiveFirewallPolicy",
     "RegionSetLabelsRequest",
     "RegionSetPolicyRequest",
     "RegionTargetHttpsProxiesSetSslCertificatesRequest",
     "RegionUrlMapsValidateRequest",
     "RemoveAssociationFirewallPolicyRequest",
+    "RemoveAssociationNetworkFirewallPolicyRequest",
+    "RemoveAssociationRegionNetworkFirewallPolicyRequest",
     "RemoveHealthCheckTargetPoolRequest",
     "RemoveInstancesInstanceGroupRequest",
     "RemoveInstanceTargetPoolRequest",
@@ -2023,6 +2180,8 @@ __all__ = (
     "RemoveResourcePoliciesInstanceRequest",
     "RemoveResourcePoliciesRegionDiskRequest",
     "RemoveRuleFirewallPolicyRequest",
+    "RemoveRuleNetworkFirewallPolicyRequest",
+    "RemoveRuleRegionNetworkFirewallPolicyRequest",
     "RemoveRuleSecurityPolicyRequest",
     "RequestMirrorPolicy",
     "Reservation",
@@ -2056,6 +2215,7 @@ __all__ = (
     "ResourcePolicySnapshotSchedulePolicySnapshotProperties",
     "ResourcePolicyWeeklyCycle",
     "ResourcePolicyWeeklyCycleDayOfWeek",
+    "ResumeInstanceRequest",
     "Route",
     "RouteAsPath",
     "RouteList",
@@ -2080,22 +2240,33 @@ __all__ = (
     "RouterStatusNatStatusNatRuleStatus",
     "RouterStatusResponse",
     "Rule",
+    "SavedAttachedDisk",
+    "SavedDisk",
     "ScalingScheduleStatus",
     "Scheduling",
     "SchedulingNodeAffinity",
     "ScratchDisks",
     "Screenshot",
+    "SecurityPoliciesAggregatedList",
     "SecurityPoliciesListPreconfiguredExpressionSetsResponse",
+    "SecurityPoliciesScopedList",
     "SecurityPoliciesWafConfig",
     "SecurityPolicy",
     "SecurityPolicyAdaptiveProtectionConfig",
     "SecurityPolicyAdaptiveProtectionConfigLayer7DdosDefenseConfig",
     "SecurityPolicyAdvancedOptionsConfig",
+    "SecurityPolicyDdosProtectionConfig",
     "SecurityPolicyList",
+    "SecurityPolicyRecaptchaOptionsConfig",
     "SecurityPolicyReference",
     "SecurityPolicyRule",
+    "SecurityPolicyRuleHttpHeaderAction",
+    "SecurityPolicyRuleHttpHeaderActionHttpHeaderOption",
     "SecurityPolicyRuleMatcher",
     "SecurityPolicyRuleMatcherConfig",
+    "SecurityPolicyRuleRateLimitOptions",
+    "SecurityPolicyRuleRateLimitOptionsThreshold",
+    "SecurityPolicyRuleRedirectOptions",
     "SecuritySettings",
     "SendDiagnosticInterruptInstanceRequest",
     "SendDiagnosticInterruptInstanceResponse",
@@ -2115,15 +2286,20 @@ __all__ = (
     "SetDefaultNetworkTierProjectRequest",
     "SetDeletionProtectionInstanceRequest",
     "SetDiskAutoDeleteInstanceRequest",
+    "SetEdgeSecurityPolicyBackendBucketRequest",
+    "SetEdgeSecurityPolicyBackendServiceRequest",
     "SetIamPolicyDiskRequest",
     "SetIamPolicyFirewallPolicyRequest",
     "SetIamPolicyImageRequest",
     "SetIamPolicyInstanceRequest",
     "SetIamPolicyInstanceTemplateRequest",
     "SetIamPolicyLicenseRequest",
+    "SetIamPolicyMachineImageRequest",
+    "SetIamPolicyNetworkFirewallPolicyRequest",
     "SetIamPolicyNodeGroupRequest",
     "SetIamPolicyNodeTemplateRequest",
     "SetIamPolicyRegionDiskRequest",
+    "SetIamPolicyRegionNetworkFirewallPolicyRequest",
     "SetIamPolicyReservationRequest",
     "SetIamPolicyResourcePolicyRequest",
     "SetIamPolicyServiceAttachmentRequest",
@@ -2170,6 +2346,8 @@ __all__ = (
     "SetUrlMapTargetHttpProxyRequest",
     "SetUrlMapTargetHttpsProxyRequest",
     "SetUsageExportBucketProjectRequest",
+    "ShareSettings",
+    "ShareSettingsProjectConfig",
     "ShieldedInstanceConfig",
     "ShieldedInstanceIdentity",
     "ShieldedInstanceIdentityEntry",
@@ -2178,7 +2356,9 @@ __all__ = (
     "SimulateMaintenanceEventInstanceRequest",
     "Snapshot",
     "SnapshotList",
+    "SourceDiskEncryptionKey",
     "SourceInstanceParams",
+    "SourceInstanceProperties",
     "SslCertificate",
     "SslCertificateAggregatedList",
     "SslCertificateList",
@@ -2205,6 +2385,7 @@ __all__ = (
     "SubnetworksScopedList",
     "SubnetworksSetPrivateIpGoogleAccessRequest",
     "Subsetting",
+    "SuspendInstanceRequest",
     "SwitchToCustomModeNetworkRequest",
     "Tags",
     "TargetGrpcProxy",
@@ -2256,11 +2437,14 @@ __all__ = (
     "TestIamPermissionsInstanceTemplateRequest",
     "TestIamPermissionsLicenseCodeRequest",
     "TestIamPermissionsLicenseRequest",
+    "TestIamPermissionsMachineImageRequest",
     "TestIamPermissionsNetworkEndpointGroupRequest",
+    "TestIamPermissionsNetworkFirewallPolicyRequest",
     "TestIamPermissionsNodeGroupRequest",
     "TestIamPermissionsNodeTemplateRequest",
     "TestIamPermissionsPacketMirroringRequest",
     "TestIamPermissionsRegionDiskRequest",
+    "TestIamPermissionsRegionNetworkFirewallPolicyRequest",
     "TestIamPermissionsReservationRequest",
     "TestIamPermissionsResourcePolicyRequest",
     "TestIamPermissionsServiceAttachmentRequest",
@@ -2284,8 +2468,10 @@ __all__ = (
     "UpdatePerInstanceConfigsRegionInstanceGroupManagerRequest",
     "UpdateRegionAutoscalerRequest",
     "UpdateRegionBackendServiceRequest",
+    "UpdateRegionCommitmentRequest",
     "UpdateRegionHealthCheckRequest",
     "UpdateRegionUrlMapRequest",
+    "UpdateReservationRequest",
     "UpdateRouterRequest",
     "UpdateShieldedInstanceConfigInstanceRequest",
     "UpdateUrlMapRequest",
