@@ -30,6 +30,8 @@ from requests.sessions import Session
 
 from google.api_core import client_options
 from google.api_core import exceptions as core_exceptions
+from google.api_core import extended_operation  # type: ignore
+from google.api_core import future
 from google.api_core import gapic_v1
 from google.api_core import grpc_helpers
 from google.api_core import grpc_helpers_async
@@ -531,6 +533,645 @@ def test_region_instances_client_client_options_credentials_file(
         )
 
 
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        compute.BulkInsertRegionInstanceRequest,
+        dict,
+    ],
+)
+def test_bulk_insert_rest(request_type):
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project": "sample1", "region": "sample2"}
+    request_init["bulk_insert_instance_resource_resource"] = {
+        "count": 553,
+        "instance_properties": {
+            "advanced_machine_features": {
+                "enable_nested_virtualization": True,
+                "enable_uefi_networking": True,
+                "threads_per_core": 1689,
+            },
+            "can_ip_forward": True,
+            "confidential_instance_config": {"enable_confidential_compute": True},
+            "description": "description_value",
+            "disks": [
+                {
+                    "auto_delete": True,
+                    "boot": True,
+                    "device_name": "device_name_value",
+                    "disk_encryption_key": {
+                        "kms_key_name": "kms_key_name_value",
+                        "kms_key_service_account": "kms_key_service_account_value",
+                        "raw_key": "raw_key_value",
+                        "rsa_encrypted_key": "rsa_encrypted_key_value",
+                        "sha256": "sha256_value",
+                    },
+                    "disk_size_gb": 1261,
+                    "guest_os_features": [{"type_": "type__value"}],
+                    "index": 536,
+                    "initialize_params": {
+                        "description": "description_value",
+                        "disk_name": "disk_name_value",
+                        "disk_size_gb": 1261,
+                        "disk_type": "disk_type_value",
+                        "labels": {},
+                        "licenses": ["licenses_value_1", "licenses_value_2"],
+                        "on_update_action": "on_update_action_value",
+                        "provisioned_iops": 1740,
+                        "resource_policies": [
+                            "resource_policies_value_1",
+                            "resource_policies_value_2",
+                        ],
+                        "source_image": "source_image_value",
+                        "source_image_encryption_key": {},
+                        "source_snapshot": "source_snapshot_value",
+                        "source_snapshot_encryption_key": {},
+                    },
+                    "interface": "interface_value",
+                    "kind": "kind_value",
+                    "licenses": ["licenses_value_1", "licenses_value_2"],
+                    "mode": "mode_value",
+                    "shielded_instance_initial_state": {
+                        "dbs": [
+                            {"content": "content_value", "file_type": "file_type_value"}
+                        ],
+                        "dbxs": {},
+                        "keks": {},
+                        "pk": {},
+                    },
+                    "source": "source_value",
+                    "type_": "type__value",
+                }
+            ],
+            "guest_accelerators": [
+                {
+                    "accelerator_count": 1805,
+                    "accelerator_type": "accelerator_type_value",
+                }
+            ],
+            "labels": {},
+            "machine_type": "machine_type_value",
+            "metadata": {
+                "fingerprint": "fingerprint_value",
+                "items": [{"key": "key_value", "value": "value_value"}],
+                "kind": "kind_value",
+            },
+            "min_cpu_platform": "min_cpu_platform_value",
+            "network_interfaces": [
+                {
+                    "access_configs": [
+                        {
+                            "external_ipv6": "external_ipv6_value",
+                            "external_ipv6_prefix_length": 2837,
+                            "kind": "kind_value",
+                            "name": "name_value",
+                            "nat_i_p": "nat_i_p_value",
+                            "network_tier": "network_tier_value",
+                            "public_ptr_domain_name": "public_ptr_domain_name_value",
+                            "set_public_ptr": True,
+                            "type_": "type__value",
+                        }
+                    ],
+                    "alias_ip_ranges": [
+                        {
+                            "ip_cidr_range": "ip_cidr_range_value",
+                            "subnetwork_range_name": "subnetwork_range_name_value",
+                        }
+                    ],
+                    "fingerprint": "fingerprint_value",
+                    "internal_ipv6_prefix_length": 2831,
+                    "ipv6_access_configs": {},
+                    "ipv6_access_type": "ipv6_access_type_value",
+                    "ipv6_address": "ipv6_address_value",
+                    "kind": "kind_value",
+                    "name": "name_value",
+                    "network": "network_value",
+                    "network_i_p": "network_i_p_value",
+                    "nic_type": "nic_type_value",
+                    "queue_count": 1197,
+                    "stack_type": "stack_type_value",
+                    "subnetwork": "subnetwork_value",
+                }
+            ],
+            "network_performance_config": {
+                "total_egress_bandwidth_tier": "total_egress_bandwidth_tier_value"
+            },
+            "private_ipv6_google_access": "private_ipv6_google_access_value",
+            "reservation_affinity": {
+                "consume_reservation_type": "consume_reservation_type_value",
+                "key": "key_value",
+                "values": ["values_value_1", "values_value_2"],
+            },
+            "resource_manager_tags": {},
+            "resource_policies": [
+                "resource_policies_value_1",
+                "resource_policies_value_2",
+            ],
+            "scheduling": {
+                "automatic_restart": True,
+                "instance_termination_action": "instance_termination_action_value",
+                "location_hint": "location_hint_value",
+                "min_node_cpus": 1379,
+                "node_affinities": [
+                    {
+                        "key": "key_value",
+                        "operator": "operator_value",
+                        "values": ["values_value_1", "values_value_2"],
+                    }
+                ],
+                "on_host_maintenance": "on_host_maintenance_value",
+                "preemptible": True,
+                "provisioning_model": "provisioning_model_value",
+            },
+            "service_accounts": [
+                {"email": "email_value", "scopes": ["scopes_value_1", "scopes_value_2"]}
+            ],
+            "shielded_instance_config": {
+                "enable_integrity_monitoring": True,
+                "enable_secure_boot": True,
+                "enable_vtpm": True,
+            },
+            "tags": {
+                "fingerprint": "fingerprint_value",
+                "items": ["items_value_1", "items_value_2"],
+            },
+        },
+        "location_policy": {"locations": {}},
+        "min_count": 972,
+        "name_pattern": "name_pattern_value",
+        "per_instance_properties": {},
+        "source_instance_template": "source_instance_template_value",
+    }
+    request = request_type(request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = compute.Operation(
+            client_operation_id="client_operation_id_value",
+            creation_timestamp="creation_timestamp_value",
+            description="description_value",
+            end_time="end_time_value",
+            http_error_message="http_error_message_value",
+            http_error_status_code=2374,
+            id=205,
+            insert_time="insert_time_value",
+            kind="kind_value",
+            name="name_value",
+            operation_group_id="operation_group_id_value",
+            operation_type="operation_type_value",
+            progress=885,
+            region="region_value",
+            self_link="self_link_value",
+            start_time="start_time_value",
+            status=compute.Operation.Status.DONE,
+            status_message="status_message_value",
+            target_id=947,
+            target_link="target_link_value",
+            user="user_value",
+            zone="zone_value",
+        )
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        json_return_value = compute.Operation.to_json(return_value)
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.bulk_insert(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, extended_operation.ExtendedOperation)
+    assert response.client_operation_id == "client_operation_id_value"
+    assert response.creation_timestamp == "creation_timestamp_value"
+    assert response.description == "description_value"
+    assert response.end_time == "end_time_value"
+    assert response.http_error_message == "http_error_message_value"
+    assert response.http_error_status_code == 2374
+    assert response.id == 205
+    assert response.insert_time == "insert_time_value"
+    assert response.kind == "kind_value"
+    assert response.name == "name_value"
+    assert response.operation_group_id == "operation_group_id_value"
+    assert response.operation_type == "operation_type_value"
+    assert response.progress == 885
+    assert response.region == "region_value"
+    assert response.self_link == "self_link_value"
+    assert response.start_time == "start_time_value"
+    assert response.status == compute.Operation.Status.DONE
+    assert response.status_message == "status_message_value"
+    assert response.target_id == 947
+    assert response.target_link == "target_link_value"
+    assert response.user == "user_value"
+    assert response.zone == "zone_value"
+
+
+def test_bulk_insert_rest_required_fields(
+    request_type=compute.BulkInsertRegionInstanceRequest,
+):
+    transport_class = transports.RegionInstancesRestTransport
+
+    request_init = {}
+    request_init["project"] = ""
+    request_init["region"] = ""
+    request = request_type(request_init)
+    jsonified_request = json.loads(
+        request_type.to_json(
+            request, including_default_value_fields=False, use_integers_for_enums=False
+        )
+    )
+
+    # verify fields with default values are dropped
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).bulk_insert._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with default values are now present
+
+    jsonified_request["project"] = "project_value"
+    jsonified_request["region"] = "region_value"
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).bulk_insert._get_unset_required_fields(jsonified_request)
+    # Check that path parameters and body parameters are not mixing in.
+    assert not set(unset_fields) - set(("request_id",))
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with non-default values are left alone
+    assert "project" in jsonified_request
+    assert jsonified_request["project"] == "project_value"
+    assert "region" in jsonified_request
+    assert jsonified_request["region"] == "region_value"
+
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+    request = request_type(request_init)
+
+    # Designate an appropriate value for the returned response.
+    return_value = compute.Operation()
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(Session, "request") as req:
+        # We need to mock transcode() because providing default values
+        # for required fields will fail the real version if the http_options
+        # expect actual values for those fields.
+        with mock.patch.object(path_template, "transcode") as transcode:
+            # A uri without fields and an empty body will force all the
+            # request fields to show up in the query_params.
+            transcode_result = {
+                "uri": "v1/sample_method",
+                "method": "post",
+                "query_params": request_init,
+            }
+            transcode_result["body"] = {}
+            transcode.return_value = transcode_result
+
+            response_value = Response()
+            response_value.status_code = 200
+            json_return_value = compute.Operation.to_json(return_value)
+            response_value._content = json_return_value.encode("UTF-8")
+            req.return_value = response_value
+
+            response = client.bulk_insert(request)
+
+            expected_params = []
+            actual_params = req.call_args.kwargs["params"]
+            assert expected_params == actual_params
+
+
+def test_bulk_insert_rest_unset_required_fields():
+    transport = transports.RegionInstancesRestTransport(
+        credentials=ga_credentials.AnonymousCredentials
+    )
+
+    unset_fields = transport.bulk_insert._get_unset_required_fields({})
+    assert set(unset_fields) == (
+        set(("requestId",))
+        & set(
+            (
+                "bulkInsertInstanceResourceResource",
+                "project",
+                "region",
+            )
+        )
+    )
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_bulk_insert_rest_interceptors(null_interceptor):
+    transport = transports.RegionInstancesRestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.RegionInstancesRestInterceptor(),
+    )
+    client = RegionInstancesClient(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.RegionInstancesRestInterceptor, "post_bulk_insert"
+    ) as post, mock.patch.object(
+        transports.RegionInstancesRestInterceptor, "pre_bulk_insert"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": None,
+            "query_params": {},
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = compute.Operation.to_json(compute.Operation())
+
+        request = compute.BulkInsertRegionInstanceRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = compute.Operation
+
+        client.bulk_insert(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_bulk_insert_rest_bad_request(
+    transport: str = "rest", request_type=compute.BulkInsertRegionInstanceRequest
+):
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"project": "sample1", "region": "sample2"}
+    request_init["bulk_insert_instance_resource_resource"] = {
+        "count": 553,
+        "instance_properties": {
+            "advanced_machine_features": {
+                "enable_nested_virtualization": True,
+                "enable_uefi_networking": True,
+                "threads_per_core": 1689,
+            },
+            "can_ip_forward": True,
+            "confidential_instance_config": {"enable_confidential_compute": True},
+            "description": "description_value",
+            "disks": [
+                {
+                    "auto_delete": True,
+                    "boot": True,
+                    "device_name": "device_name_value",
+                    "disk_encryption_key": {
+                        "kms_key_name": "kms_key_name_value",
+                        "kms_key_service_account": "kms_key_service_account_value",
+                        "raw_key": "raw_key_value",
+                        "rsa_encrypted_key": "rsa_encrypted_key_value",
+                        "sha256": "sha256_value",
+                    },
+                    "disk_size_gb": 1261,
+                    "guest_os_features": [{"type_": "type__value"}],
+                    "index": 536,
+                    "initialize_params": {
+                        "description": "description_value",
+                        "disk_name": "disk_name_value",
+                        "disk_size_gb": 1261,
+                        "disk_type": "disk_type_value",
+                        "labels": {},
+                        "licenses": ["licenses_value_1", "licenses_value_2"],
+                        "on_update_action": "on_update_action_value",
+                        "provisioned_iops": 1740,
+                        "resource_policies": [
+                            "resource_policies_value_1",
+                            "resource_policies_value_2",
+                        ],
+                        "source_image": "source_image_value",
+                        "source_image_encryption_key": {},
+                        "source_snapshot": "source_snapshot_value",
+                        "source_snapshot_encryption_key": {},
+                    },
+                    "interface": "interface_value",
+                    "kind": "kind_value",
+                    "licenses": ["licenses_value_1", "licenses_value_2"],
+                    "mode": "mode_value",
+                    "shielded_instance_initial_state": {
+                        "dbs": [
+                            {"content": "content_value", "file_type": "file_type_value"}
+                        ],
+                        "dbxs": {},
+                        "keks": {},
+                        "pk": {},
+                    },
+                    "source": "source_value",
+                    "type_": "type__value",
+                }
+            ],
+            "guest_accelerators": [
+                {
+                    "accelerator_count": 1805,
+                    "accelerator_type": "accelerator_type_value",
+                }
+            ],
+            "labels": {},
+            "machine_type": "machine_type_value",
+            "metadata": {
+                "fingerprint": "fingerprint_value",
+                "items": [{"key": "key_value", "value": "value_value"}],
+                "kind": "kind_value",
+            },
+            "min_cpu_platform": "min_cpu_platform_value",
+            "network_interfaces": [
+                {
+                    "access_configs": [
+                        {
+                            "external_ipv6": "external_ipv6_value",
+                            "external_ipv6_prefix_length": 2837,
+                            "kind": "kind_value",
+                            "name": "name_value",
+                            "nat_i_p": "nat_i_p_value",
+                            "network_tier": "network_tier_value",
+                            "public_ptr_domain_name": "public_ptr_domain_name_value",
+                            "set_public_ptr": True,
+                            "type_": "type__value",
+                        }
+                    ],
+                    "alias_ip_ranges": [
+                        {
+                            "ip_cidr_range": "ip_cidr_range_value",
+                            "subnetwork_range_name": "subnetwork_range_name_value",
+                        }
+                    ],
+                    "fingerprint": "fingerprint_value",
+                    "internal_ipv6_prefix_length": 2831,
+                    "ipv6_access_configs": {},
+                    "ipv6_access_type": "ipv6_access_type_value",
+                    "ipv6_address": "ipv6_address_value",
+                    "kind": "kind_value",
+                    "name": "name_value",
+                    "network": "network_value",
+                    "network_i_p": "network_i_p_value",
+                    "nic_type": "nic_type_value",
+                    "queue_count": 1197,
+                    "stack_type": "stack_type_value",
+                    "subnetwork": "subnetwork_value",
+                }
+            ],
+            "network_performance_config": {
+                "total_egress_bandwidth_tier": "total_egress_bandwidth_tier_value"
+            },
+            "private_ipv6_google_access": "private_ipv6_google_access_value",
+            "reservation_affinity": {
+                "consume_reservation_type": "consume_reservation_type_value",
+                "key": "key_value",
+                "values": ["values_value_1", "values_value_2"],
+            },
+            "resource_manager_tags": {},
+            "resource_policies": [
+                "resource_policies_value_1",
+                "resource_policies_value_2",
+            ],
+            "scheduling": {
+                "automatic_restart": True,
+                "instance_termination_action": "instance_termination_action_value",
+                "location_hint": "location_hint_value",
+                "min_node_cpus": 1379,
+                "node_affinities": [
+                    {
+                        "key": "key_value",
+                        "operator": "operator_value",
+                        "values": ["values_value_1", "values_value_2"],
+                    }
+                ],
+                "on_host_maintenance": "on_host_maintenance_value",
+                "preemptible": True,
+                "provisioning_model": "provisioning_model_value",
+            },
+            "service_accounts": [
+                {"email": "email_value", "scopes": ["scopes_value_1", "scopes_value_2"]}
+            ],
+            "shielded_instance_config": {
+                "enable_integrity_monitoring": True,
+                "enable_secure_boot": True,
+                "enable_vtpm": True,
+            },
+            "tags": {
+                "fingerprint": "fingerprint_value",
+                "items": ["items_value_1", "items_value_2"],
+            },
+        },
+        "location_policy": {"locations": {}},
+        "min_count": 972,
+        "name_pattern": "name_pattern_value",
+        "per_instance_properties": {},
+        "source_instance_template": "source_instance_template_value",
+    }
+    request = request_type(request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.bulk_insert(request)
+
+
+def test_bulk_insert_rest_flattened():
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = compute.Operation()
+
+        # get arguments that satisfy an http rule for this method
+        sample_request = {"project": "sample1", "region": "sample2"}
+
+        # get truthy value for each flattened field
+        mock_args = dict(
+            project="project_value",
+            region="region_value",
+            bulk_insert_instance_resource_resource=compute.BulkInsertInstanceResource(
+                count=553
+            ),
+        )
+        mock_args.update(sample_request)
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        json_return_value = compute.Operation.to_json(return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+
+        client.bulk_insert(**mock_args)
+
+        # Establish that the underlying call was made with the expected
+        # request object values.
+        assert len(req.mock_calls) == 1
+        _, args, _ = req.mock_calls[0]
+        assert path_template.validate(
+            "%s/compute/v1/projects/{project}/regions/{region}/instances/bulkInsert"
+            % client.transport._host,
+            args[1],
+        )
+
+
+def test_bulk_insert_rest_flattened_error(transport: str = "rest"):
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # Attempting to call a method with both a request object and flattened
+    # fields is an error.
+    with pytest.raises(ValueError):
+        client.bulk_insert(
+            compute.BulkInsertRegionInstanceRequest(),
+            project="project_value",
+            region="region_value",
+            bulk_insert_instance_resource_resource=compute.BulkInsertInstanceResource(
+                count=553
+            ),
+        )
+
+
+def test_bulk_insert_rest_error():
+    client = RegionInstancesClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
 @pytest.mark.parametrize(
     "request_type",
     [
@@ -745,28 +1386,6 @@ def test_bulk_insert_unary_rest(request_type):
 
     # Establish that the response is the type that we expect.
     assert isinstance(response, compute.Operation)
-    assert response.client_operation_id == "client_operation_id_value"
-    assert response.creation_timestamp == "creation_timestamp_value"
-    assert response.description == "description_value"
-    assert response.end_time == "end_time_value"
-    assert response.http_error_message == "http_error_message_value"
-    assert response.http_error_status_code == 2374
-    assert response.id == 205
-    assert response.insert_time == "insert_time_value"
-    assert response.kind == "kind_value"
-    assert response.name == "name_value"
-    assert response.operation_group_id == "operation_group_id_value"
-    assert response.operation_type == "operation_type_value"
-    assert response.progress == 885
-    assert response.region == "region_value"
-    assert response.self_link == "self_link_value"
-    assert response.start_time == "start_time_value"
-    assert response.status == compute.Operation.Status.DONE
-    assert response.status_message == "status_message_value"
-    assert response.target_id == 947
-    assert response.target_link == "target_link_value"
-    assert response.user == "user_value"
-    assert response.zone == "zone_value"
 
 
 def test_bulk_insert_unary_rest_required_fields(
@@ -1245,6 +1864,19 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "rest",
+    ],
+)
+def test_transport_kind(transport_name):
+    transport = RegionInstancesClient.get_transport_class(transport_name)(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert transport.kind == transport_name
+
+
 def test_region_instances_base_transport_error():
     # Passing both a credentials object and credentials_file should raise an error
     with pytest.raises(core_exceptions.DuplicateCredentialArgs):
@@ -1274,6 +1906,14 @@ def test_region_instances_base_transport():
     with pytest.raises(NotImplementedError):
         transport.close()
 
+    # Catch all for all remaining methods and properties
+    remainder = [
+        "kind",
+    ]
+    for r in remainder:
+        with pytest.raises(NotImplementedError):
+            getattr(transport, r)()
+
 
 def test_region_instances_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
